@@ -5,9 +5,17 @@ import heapq
 from zephyrus_sc2_parser.s2protocol_fixed import versions
 from zephyrus_sc2_parser.game.game import Game
 from zephyrus_sc2_parser.game.player_state import PlayerState
-from zephyrus_sc2_parser.utils import create_event, create_players, convert_time
+from zephyrus_sc2_parser.utils import (
+    import_gamedata, get_map_info, create_event, create_players, convert_time
+)
 import logging
 
+is_engagement_simulator_installed = False
+try:
+    from sc2_simulator import simulate_engagement
+    is_engagement_simulator_installed = True
+except ImportError:
+    pass
 
 non_english_maps = {
     b'\xec\x95\x84\xed\x81\xac\xeb\xa1\x9c\xed\x8f\xb4\xeb\xa6\xac\xec\x8a\xa4 \x2d \xeb\x9e\x98\xeb\x8d\x94': 'Acropolis LE',
@@ -23,7 +31,6 @@ non_english_maps = {
 def initial_summary_stats(game, metadata, detailed_info, local=False):
     summary_stats = {
         'mmr': {1: 0, 2: 0},
-        'mmr_diff': {1: 0, 2: 0},
         'avg_resource_collection_rate': {
             'minerals': {1: 0, 2: 0},
             'gas': {1: 0, 2: 0}
@@ -33,19 +40,25 @@ def initial_summary_stats(game, metadata, detailed_info, local=False):
             'gas': {1: 0, 2: 0}
         },
         'apm': {1: 0, 2: 0},
+        'spm': {1: 0, 2: 0},
         'resources_lost': {
             'minerals': {1: 0, 2: 0},
             'gas': {1: 0, 2: 0}
         },
+        'resources_collected': {
+            'minerals': {1: 0, 2: 0},
+            'gas': {1: 0, 2: 0},
+        },
         'workers_produced': {1: 0, 2: 0},
         'workers_killed': {1: 0, 2: 0},
         'workers_lost': {1: 0, 2: 0},
-        'inject_count': {1: 0, 2: 0},
+        'supply_block': {1: 0, 2: 0},
         'sq': {1: 0, 2: 0},
         'avg_pac_per_min': {1: 0, 2: 0},
         'avg_pac_action_latency': {1: 0, 2: 0},
         'avg_pac_actions': {1: 0, 2: 0},
         'avg_pac_gap': {1: 0, 2: 0},
+        'race': {1: {}, 2: {}},
     }
 
     mmr_data = detailed_info['m_syncLobbyState']['m_userInitialData']
@@ -54,15 +67,7 @@ def initial_summary_stats(game, metadata, detailed_info, local=False):
         if not local:
             return None
 
-    ranked_game = False
-    player1_mmr = mmr_data[0]['m_scaledRating']
-    player2_mmr = mmr_data[1]['m_scaledRating']
     for p in metadata['Players']:
-        if player1_mmr and player2_mmr:
-            ranked_game = True
-        else:
-            ranked_game = False
-
         if p['Result'] == 'Win':
             game.winner = p['PlayerID']
 
@@ -76,10 +81,6 @@ def initial_summary_stats(game, metadata, detailed_info, local=False):
         else:
             summary_stats['mmr'][player_id] = 0
 
-    if ranked_game:
-        summary_stats['mmr_diff'][1] = player1_mmr-player2_mmr
-        summary_stats['mmr_diff'][2] = player2_mmr-player1_mmr
-
     return summary_stats
 
 
@@ -130,6 +131,11 @@ def setup(filename):
     events = heapq.merge(game_events, tracker_events, key=lambda x: x['_gameloop'])
     events = sorted(events, key=lambda x: x['_gameloop'])
 
+    # for event in events:
+    #     # if event['_event'] == 'NNet.Game.SCmdEvent':  # == 'NNet.Replay.Tracker.SUnitPositionsEvent':
+    #     print(event)
+    #     print('\n')
+
     for event in events:
         if event['_event'] == 'NNet.Game.SGameUserLeaveEvent':
             game_length = event['_gameloop']
@@ -141,46 +147,52 @@ def setup(filename):
 def parse_replay(filename, *, local=False, detailed=False):
     try:
         events, player_info, detailed_info, metadata, game_length, protocol = setup(filename)
-
         if events is None:
             logging.critical('Aborting replay due to bad decode...')
-            return None, None, None, None
+            return None, None, None, None, None
 
         players = create_players(player_info, events)
     except ValueError as error:
         logging.critical('A ValueError occured:', error, 'unreadable header')
-        return None, None, None, None
+        return None, None, None, None, None
     except ImportError as error:
         logging.critical('An ImportError occured:', error, 'unsupported protocol')
-        return None, None, None, None
+        return None, None, None, None, None
     except KeyError as error:
         logging.critical('A KeyError error occured:', error, 'unreadable file info')
-        return None, None, None, None
+        return None, None, None, None, None
+
+    if not players:
+        logging.debug('Aborting replay due to <2 players...')
+        return None, None, None, None, None
 
     if player_info['m_title'] in non_english_maps:
         game_map = non_english_maps[player_info['m_title']]
     else:
         game_map = player_info['m_title'].decode('utf-8')
-    played_at = convert_time(player_info['m_timeUTC'])
 
-    if not players:
-        logging.debug('Aborting replay due to <2 players...')
-        return None, None, None, None
+    played_at = convert_time(player_info['m_timeUTC'])
+    map_info = get_map_info(player_info, game_map)
+
+    if not map_info:
+        logging.debug('Aborting replay due to missing map data')
+        return None, None, None, None, None
 
     current_game = Game(
         players,
-        game_map,
+        map_info,
         played_at,
         game_length,
         events,
-        protocol
+        protocol,
+        import_gamedata(protocol),
     )
 
     summary_stats = initial_summary_stats(current_game, metadata, detailed_info, local)
 
     if summary_stats is None:
         logging.debug('Aborting replay due to missing MMR value(s)')
-        return None, None, None, None
+        return None, None, None, None, None
 
     action_events = [
         'NNet.Game.SControlGroupUpdateEvent',
@@ -195,9 +207,6 @@ def parse_replay(filename, *, local=False, detailed=False):
         if gameloop <= game_length:
             current_event = create_event(current_game, event, protocol, summary_stats)
             if current_event:
-                # print(event)
-                # print('\n')
-
                 result = current_event.parse_event()
 
                 if result:
@@ -207,15 +216,16 @@ def parse_replay(filename, *, local=False, detailed=False):
                     current_event.player.current_pac.actions.append(gameloop)
 
                 # 112 = 5sec of game time
-                players.sort()
                 if gameloop >= current_tick or gameloop == game_length:
                     player1_state = PlayerState(
-                        players[0],
+                        current_game,
+                        players[1],
                         gameloop,
                     )
 
                     player2_state = PlayerState(
-                        players[1],
+                        current_game,
+                        players[2],
                         gameloop,
                     )
 
@@ -225,20 +235,110 @@ def parse_replay(filename, *, local=False, detailed=False):
                         2: player2_state.summary
                     })
 
+                    player_units = {1: [], 2: []}
+                    for p_id, p in players.items():
+                        for obj in p.objects.values():
+                            if obj.status == 'live':
+                                player_units[p_id].append(obj.name)
+
+                    current_game.engagements.append((
+                        player_units[1],
+                        player_units[2],
+                        players[1].upgrades,
+                        players[2].upgrades,
+                        gameloop,
+                    ))
+
                     current_tick += 112
 
-    # for player in players:
-    #     print(player.name)
-    #     for k, v in current_game.state[-1][player.player_id - 1].summary['unit'].items():
-    #         print(k, v)
+    engagement_analysis = []
+    if is_engagement_simulator_installed:
+        engagement_outcomes = simulate_engagement(current_game.engagements)
+        for winner, unit_health, gameloop in engagement_outcomes:
+            total_health = {1: (0, 0), 2: (0, 0)}
+            for unit in unit_health:
+                total_health[unit[0]] = (
+                    total_health[unit[0]][0] + unit[2],
+                    total_health[unit[0]][1] + unit[3],
+                )
 
-    #     for k, v in current_game.state[-1][player.player_id - 1].summary['building'].items():
-    #         print(k, v)
-    #     print('\n')
+            if total_health[1][1] > 0 and total_health[2][1] > 0:
+                engagement_analysis.append({
+                    'winner': winner,
+                    'health': total_health[winner],
+                    'remaining_health':  round((total_health[winner][0] / total_health[winner][1]), 3),
+                    'gameloop': gameloop,
+                })
 
     players_export = {}
-    for player in players:
+    for p_id, player in players.items():
         summary_stats = player.calc_pac(summary_stats, game_length)
+        summary_stats['spm'][player.player_id] = player.calc_spm(current_game.game_length)
+
+        opp_id = 1 if p_id == 2 else 2
+
+        if player.race == 'Zerg':
+            print(current_game.timeline[-1][player.player_id]['race'])
+            summary_stats['race'][player.player_id]['inject_efficiency'] = current_game.timeline[-1][player.player_id]['race']['inject_efficiency']
+            summary_stats['race'][player.player_id]['avg_idle_larva'] = round(sum(player.idle_larva) / len(player.idle_larva), 1)
+            summary_stats['race'][player.player_id]['creep'] = current_game.timeline[-1][player.player_id]['race']['creep']
+
+        elif player.race == 'Protoss':
+            opp_player = players[opp_id]
+
+            protoss_splash = {
+                'HighTemplar': (0, 0),
+                'Disruptor': (0, 0),
+                'Colossus': (0, 0),
+            }
+
+            units = current_game.gamedata['units']
+
+            for obj in opp_player.objects.values():
+                if obj.killed_by and obj.killed_by.name in protoss_splash:
+                    protoss_splash[obj.killed_by] = (
+                        protoss_splash[obj.killed_by][0] + units[obj.name]['mineral_cost'],
+                        protoss_splash[obj.killed_by][1] + units[obj.name]['gas_cost'],
+                    )
+
+            splash_efficiency = {}
+            for splash_unit, resources_killed in protoss_splash.items():
+                # if not default
+                if resources_killed != (0, 0):
+                    unit_mineral_cost = units[splash_unit]['mineral_cost']
+                    unit_gas_cost = units[splash_unit]['gas_cost']
+
+                    splash_efficiency[splash_unit] = (
+                        round(resources_killed[0] / unit_mineral_cost, 2),
+                        round(resources_killed[1] / unit_gas_cost, 2),
+                    )
+
+            if splash_efficiency:
+                summary_stats['race'][player.player_id]['splash_efficiency'] = splash_efficiency
+
+        if 'energy' in current_game.timeline[-1][player.player_id]['race']:
+            energy_stats = current_game.timeline[-1][player.player_id]['race']['energy']
+            energy_efficiency = {}
+            energy_idle_time = {}
+
+            for obj_name, energy_info in energy_stats.items():
+                energy_efficiency[obj_name] = []
+                energy_idle_time[obj_name] = []
+                for obj_data in energy_info:
+                    energy_efficiency[obj_name].append(obj_data[1])
+                    energy_idle_time[obj_name].append(obj_data[2])
+
+            summary_stats['race'][player.player_id]['energy'] = {
+                'efficiency': energy_efficiency,
+                'idle_time': energy_idle_time,
+            }
+
+        if 'warpgate_efficiency' in current_game.timeline[-1][player.player_id]['race']:
+            warpgate_efficiency = current_game.timeline[-1][player.player_id]['race']['warpgate_efficiency']
+            summary_stats['race'][player.player_id]['warpgate'] = {
+                'efficiency': warpgate_efficiency[0],
+                'idle_time': warpgate_efficiency[1],
+            }
 
         if not detailed:
             del player.__dict__['pac_list']
@@ -246,20 +346,23 @@ def parse_replay(filename, *, local=False, detailed=False):
             del player.__dict__['objects']
             del player.__dict__['current_selection']
             del player.__dict__['control_groups']
+            del player.__dict__['warpgate_cooldowns']
+            del player.__dict__['warpgate_efficiency']
             del player.__dict__['active_ability']
             del player.__dict__['unspent_resources']
             del player.__dict__['collection_rate']
+            del player.__dict__['resources_collected']
+            del player.__dict__['supply']
+            del player.__dict__['supply_cap']
         players_export[player.player_id] = player
 
-        for p in players:
-            if p.player_id != player.player_id:
-                summary_stats['workers_killed'][p.player_id] = summary_stats['workers_lost'][player.player_id]
+        summary_stats['workers_killed'][opp_id] = summary_stats['workers_lost'][player.player_id]
 
     metadata_export = {
         'time_played_at': current_game.played_at,
-        'map': current_game.map,
+        'map': current_game.map['name'],
         'game_length': math.floor(current_game.game_length/22.4),
         'winner': current_game.winner
     }
 
-    return players_export, current_game.timeline, summary_stats, metadata_export
+    return players_export, current_game.timeline, engagement_analysis, summary_stats, metadata_export
